﻿using System;
using System.IO;
using System.Reflection;
using System.Runtime.CompilerServices;

using Mono.Cecil;

namespace MurkaInject
{
    public class Program
    {
        private static readonly string[] assemblyToInject =
            {
                // alternatively "MyAssembly, PublicKey=0024000004800000... etc."
                "UnitTestProject"
            };

        static void Main(string[] args)
        {
            if (args == null || args.Length == 0)
            {
                Assembly assembly = Assembly.GetExecutingAssembly();
                Console.WriteLine(String.Format("Injecting InternalVisibleTo\nUsing:\n {0} filename.dll",assembly.GetName().Name));
                return;
            }
            string dllFullPath = Path.GetFullPath(args[0]);
            if (!File.Exists(dllFullPath))
            {
                Console.WriteLine("File not exists {0}", dllFullPath);
                return;
            }
            ReaderParameters parameters = new ReaderParameters();
            ModuleDefinition asm = ModuleDefinition.ReadModule(dllFullPath, parameters);
            foreach (var toInject in assemblyToInject)
            {
                CustomAttribute ca =
                    new CustomAttribute(
                        asm.Import(typeof(InternalsVisibleToAttribute).GetConstructor(new[] { typeof(string) })));
                ca.ConstructorArguments.Add(new CustomAttributeArgument(asm.TypeSystem.String, toInject));
                asm.Assembly.CustomAttributes.Add(ca);
            }
            string directoryName = Path.GetDirectoryName(dllFullPath);
            string fileName = Path.GetFileNameWithoutExtension(dllFullPath);
            string extension = Path.GetExtension(dllFullPath);
            string outputDir = Path.Combine(directoryName, "Out");
            string outputFile = Path.Combine(outputDir, fileName + extension);
            Directory.CreateDirectory(outputDir);
            asm.Write(outputFile);
            // note if the assembly is strongly-signed you need to resign it like
            // asm.Write(@"c:\folder-modified\ThirdPartyAssembly.dll", new WriterParameters {
            //   StrongNameKeyPair = new StrongNameKeyPair(File.ReadAllBytes(@"c:\MyKey.snk"))
            // });
        }
    }
}

/*
 Install-Package Mono.Cecil
     */
